using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Data;

namespace SqlScriptCreator.Generators
{
    public class InsertTempTableGenerator : IScriptGenerator  
    {
        private String _tableName = "tempTable";

        [CategoryAttribute("Table settings"),
         DescriptionAttribute("The name of the generated temp table (#TABLENAME).")]
        public string TableName
        {
            get
            {
                return _tableName;
            }
            set
            {
                _tableName = value;
            }
        }

        private String _columnPrefix;

        [CategoryAttribute("Table settings"),
         DescriptionAttribute("Characters that will be prepended to every generated table column.")]
        public string ColumnPrefix
        {
            get
            {
                return _columnPrefix;
            }
            set
            {
                _columnPrefix = value;
            }
        }

        private String _columnSuffix;

        [CategoryAttribute("Table settings"),
         DescriptionAttribute("Characters that will be appended to every generated table column.")]
        public string ColumnSuffix
        {
            get
            {
                return _columnSuffix;
            }
            set
            {
                _columnSuffix = value;
            }
        }

        private String _identityColumn;

        [CategoryAttribute("Table settings"),
         DescriptionAttribute("By specifying the name of the identity column an additional column with identity properties will be defined.")]
        public String IdentityColumn
        {
            get { return _identityColumn; }
            set { _identityColumn = value; }
        }


        #region IScriptGenerator Members

        override public string ToString()
        {
            return "Insert into temp-table";
        }

        public string generateScript(String query, DataTable dtQueryResult, DataTable dtParameters)
        {
            StringBuilder sb = new StringBuilder();
            String[] fieldnames = new String[dtParameters.Rows.Count];
            String lineend;
            String lengthSpecification;

            sb.AppendLine("CREATE TABLE #" + TableName + " (");
            if (IdentityColumn != null && IdentityColumn.Length > 0)
            {
                sb.AppendLine("\t" + IdentityColumn + " INT IDENTITY(1,1),");
            }

            for (int i = 0; i < dtParameters.Rows.Count; i++)
            {
                DataRow row = dtParameters.Rows[i];
                int columnSize = Convert.ToInt32(row["ColumnSize"]);
                String dataType = row["DataTypeName"].ToString();
                String columnName = row["BaseColumnName"].ToString();

                if (i + 1 < dtParameters.Rows.Count)
                    lineend = ",";
                else
                    lineend = "";

                fieldnames[i] = ColumnPrefix + columnName + ColumnSuffix;
                sb.AppendLine("\t" + fieldnames[i] + " " + SqlTools.getDatatypeExpression(dataType, columnSize) + lineend);
            }
            sb.AppendLine(")");
            sb.AppendLine("");
            sb.Append("INSERT INTO #" + TableName + " (");
            sb.Append(String.Join(", ", fieldnames));
            sb.AppendLine(")");
            sb.AppendLine(query);
            sb.AppendLine("");
            sb.AppendLine("-- TODO: use temp table here!");
            sb.AppendLine("SELECT * FROM #"+TableName);
            sb.AppendLine("");
            sb.AppendLine("DROP TABLE #" + TableName);

            return sb.ToString();
        }

        #endregion
    }
}
